package somlib.som;

import java.io.*;
import java.util.*;

public class Vector implements Serializable
/*
 * the most basic functions of a single Vector are implemented;
 * used by the node-vectors and by each training vector
 */
{

  public static final Vector add(Vector a, Vector b) throws SOMException
  /*
   * @a + @b   is calculated
   */
  {
    if(a.dims != b.dims)
      throw new SOMException("Trying to add Vectors with different dimensions.");

    Vector v = new Vector(a.dims);
    for (int i=0;i<a.dims;i++) v.v[i] = a.v[i] + b.v[i];
    return v;
  }

  public static final Vector sub(Vector a, Vector b) throws SOMException
  /*
   * @a - @b   is calculated
   */
  {
    if(a.dims != b.dims)
      throw new SOMException("Trying to substract Vectors with different dimensions.");

    Vector v = new Vector(a.dims);
    for (int i=0;i<a.dims;i++) v.v[i] = a.v[i] - b.v[i];
    return v;
  }


  public float[] v;                // a vector's value
  public int dims = 0;              // the dimension of the vector  (v.euclLength = dims)
  public String label = null;   // the vector's name

  public Vector(int dims) {
    this.dims = dims;
    v = new float[dims];

    for(int i=0; i<dims; i++)
      v[i] = 0.0f;
  }

  public Vector(int dims, String label) {
    this.label=label;
    this.dims = dims;
    v = new float[dims];

    for(int i=0; i<dims; i++)
      v[i] = 0.0f;
  }

  public Vector(int dims, Random rnd, String label) {
    this.label=label;
    this.dims = dims;
    v = new float[dims];

    for(int i=0; i<dims; i++)
      v[i] = rnd.nextFloat();
  }

  public Vector(String str, int dims) throws SOMException {
    this.dims=dims;
    try {
      initFromString(str);
    } catch(SOMException e) {
      throw new SOMException("Vector.<init>: "+e.getMessage());
    }
  }

  public Vector(String str, String label) throws SOMException {
    this.label=label;
    this.dims = 0;
    try {
      initFromString(str);
    } catch(SOMException e) {
      throw new SOMException("Vector.<init>: "+e.getMessage());
    }
  }

  public Vector(float[] values, String label) {
    this.label=label;
    this.dims = values.length;
    this.v = values;
  }

  public Vector(Vector pattern) {
    this.label=pattern.getLabel();
    this.dims=pattern.dims;
    v = new float[dims];

    for(int i=0; i<dims; i++)
      this.v[i] = pattern.v[i];
  }

  public void initFromString(String str) throws SOMException
  /*
   * the vector's value's are initialized from @str
   */
  {
    label=new String();
    StringTokenizer strtok = new StringTokenizer(str);
    if(dims==0)
      dims=strtok.countTokens();
    v = new float[dims];
    try {
      for (int j=0; j<dims; j++) v[j]=(new Float(strtok.nextToken())).floatValue();
      if(strtok.hasMoreTokens())
        label=new String(strtok.nextToken());
      while(strtok.hasMoreTokens())
        label+=" "+strtok.nextToken();
    } catch (NumberFormatException e) {
      throw new SOMException("Cannot convert string to vector with "+dims+" dimensions: "+e.getMessage());
    }
  }

  public String returnAsString() {
    String str;
    java.lang.StringBuffer strbuf = new java.lang.StringBuffer();
    for (int j=0; j<dims; j++) strbuf.append(v[j] + " ");
    str = strbuf.toString();
    return str;
  }

  public float biggestValue()
  /*
   * returns the vector's biggest value
   */
  {
    float d=0;
    for (int i=0;i<dims;i++)
      if(v[i] > d) d=v[i];
    return d;
  }

  public float euclLength()
  /*
   * the vectors euclidean length is calculated
   */
  {
    float d=0;
    for (int i=0;i<dims;i++)
      d = d + v[i]*v[i];
    return (float)Math.sqrt(d);
  }

  public Vector normalize()
  /*
   * the vector is normalized to a (euclidean) length equal to 1.0
   */
  {
    float c = euclLength();
    if(c!=1.0 && c!=0.0)
      for (int i=0;i<dims;i++) v[i] = v[i]/c;

    return this;
  }

  public Vector scale()
  /*
   * the vector is scaled to values between 0.0 and 1.0
   */
  {
    float c = biggestValue();
    if(c!=1.0)
      for (int i=0;i<dims;i++) v[i] = v[i]/c;

    return this;
  }

  public Vector reset()
  /*
   * the vector's values are set back to 0.0
   */
  {
    for (int i=0;i<dims;i++) v[i] = 0.0f;

    return this;
  }

  public void setTo(Vector toSet)
  /*
   * the vector's values are adapted to the values of the vector @toSet
   */
  {
    v=toSet.value();
  }

  public Vector makeDimPos()
  /*
   * the vector's values are set to the corresponding positive values
   */
  {
    for (int i=0;i<dims;i++) v[i] = Math.abs(v[i]);
    return this;
  }

  public Vector mult(float f)
  /*
   * each dimension is multiplied by @f
   */
  {
    for (int i=0;i<dims;i++) v[i] = v[i]*f;
    return this;
  }

  public boolean equals(Vector v)
  /*
   * checks, whether the Vector @v has the same value
   */
  {
    for (int i=0;i<dims;i++) if (v.v[i] != this.v[i]) return false;
    return true;
  }

  public float[] value() {
    float[] result = new float[dims];
    for(int i=0; i<dims; i++)
      result[i]=v[i];
    return result;
  }

  public void setLabel(String label) {
    this.label=label;
  }

  public String getLabel() {
    if(label==null)
      return new String();
    return new String(label);
  }
}
